/******************************************************************************
*
* project name: TI-Con45
* initial date: 28/06/2000
* authors:      thomas.nussbaumer@gmx.net (coding)
                marcos.lopez@wol.es (graphics/minor improvements/BETA testing)
* description:  menu graphics and handling
*
* -----------------------------------------------------------------------------
* Revision History
* -----------------------------------------------------------------------------
*
* v1.61: 31/12/2000
*        key repeat feature added to GetUserInput
*
* v1.60: 27/12/2000
*        minor changes due to adaptions for TIGCC 0.90
*
* v1.51: 07/07/2000
*        v1.00 changed to v1.01
*
* v1.50: 03/07/2000
*        more comments added
*
* v1.20: 30/06/2000
*        (1) changed again to grayscale
*        (2) maximum width extended to 12
*
* v1.10: 29/06/2000
*        (1) changed to black/white
*        (2) menu is now drawn as simple text
*        (3) levels restricted to 4/5/6
*        (4) mode restricted to connect-4/connect-5
*        (5) width restricted to 5/6/7/8/9
*        (6) height restricted to 5/6/7
*        (7) minor size reduction
*
* v1.00: 28/06/2000
*        initial version
*
*******************************************************************************/

short selected_entry = 0;  // active menu entry

//-----------------------------------------------------------------------------
// structure used to define a menu entry
//-----------------------------------------------------------------------------
typedef struct {
    short   active;      // active option of this menu entry
    short   nr_options;  // number of options for this menu entry
    char* options[10]; // option strings of this menu entry
} menu_t;


//-----------------------------------------------------------------------------
// data of our menu
//-----------------------------------------------------------------------------

#define MAX_BOARD_WIDTH   13   // due to a strange and not found bug larger than 12
#define MAX_BOARD_HEIGHT   8   // due to a strange and not found bug larger than 8
#define MAX_BOARD_CONNECT  5

menu_t settings[5] = {
    {0, 4,{(char*)"HUMAN",(char*)"Calc1",(char*)"Calc2",(char*)"Calc3"}}, // Calc1 is level 4 !!
    {1, 4,{(char*)"HUMAN",(char*)"Calc1",(char*)"Calc2",(char*)"Calc3"}},
    {2, 8,{(char*)"5",(char*)"6",(char*)"7",(char*)"8",(char*)"9",(char*)"10",(char*)"11",(char*)"12"}},
    {1, 3,{(char*)"5",(char*)"6",(char*)"7"}},
    {0, 2,{(char*)"Connect-4",(char*)"Connect-5"}}};

char* menu_titles[] = {(char*)"Player-1",(char*)"Player-2",(char*)"Width",(char*)"Height",(char*)"Mode",(char*)"Exit"};


/*===========================================================================*/
/* get input key pressed by user                                             */
/*                                                                           */
/* replacement for ngetchx() and/or GKeyIn(), because:                       */
/*                                                                           */
/* (1) ngetchx() won't check the APD timer value                             */
/* (2) GKeyIn() destroys the display due to setting the status display       */
/*                                                                           */
/* additionally this function handles the CALCULATOR OFF combinations and    */
/* maps F1,'y','Y' to KEY_ENTER and 'n','N' to KEY_ESC                       */
/*                                                                           */
/*===========================================================================*/
short GetUserInput() {
    void           *kbq = kbd_queue();
    unsigned short  key;

    //-------------------------------------------
    // restart the APD timer now
    //-------------------------------------------
    OSTimerRestart(APD_TIMER);

    while (1) {
        //---------------------------------------
        // wait until we get a key
        // if the APD timer expires -> turn
        // calculator off
        //---------------------------------------
        while (OSdequeue(&key, kbq)) {
            if (OSTimerExpired(APD_TIMER)) {
                off();
                OSTimerRestart(APD_TIMER);
            }
        }

        //---------------------------------------
        // handle calculator-off key combinations
        // if the APD timer expires -> turn
        // calculator off
        //---------------------------------------
        if (key == 4363 || (TI89 && key == 16651) || (!TI89 && key == 8459)) {
            off();
            OSTimerRestart(APD_TIMER);

        }
        else {
            switch(key) {
                //---------------------------------------
                // treatment of F1/'Y'/'y' as KEY_ENTER
                //---------------------------------------
                case KEY_F1:
                case 'Y':
                case 'y':
                    return KEY_ENTER;
                //---------------------------------------
                // treatment of 'N'/'n' as KEY_ESC
                //---------------------------------------
                case 'N':
                case 'n':
                    return KEY_ESC;
            }
            return key & ~(unsigned short)0x800;
        }
    }
}


/*===========================================================================*/
/* fill SCR_RECT structure by taking additionally offsets for TI92 into      */
/* account                                                                   */
/*===========================================================================*/
void FillScrRect(SCR_RECT* rect,short x0,short y0,short x1,short y1) {
    rect->xy.x0 = ti92_offsetx+x0;
    rect->xy.y0 = ti92_offsety+y0;
    rect->xy.x1 = ti92_offsetx+x1;
    rect->xy.y1 = ti92_offsety+y1;
}


/*===========================================================================*/
/* invert the title of a menu entry                                          */
/*===========================================================================*/
void InvertMenuEntry() {
    SCR_RECT rect;
    short      offset = selected_entry*12;
    short      loop;

    for (loop=0;loop<2;loop++) {
        SetPlane(loop);
        FillScrRect(&rect,3,offset+15,60,offset+15+10);
        ScrRectFill(&rect,&fullscreen,A_XOR);
    }
}


/*===========================================================================*/
/* draw the value of a menu entry                                            */
/*===========================================================================*/
void DrawMenuEntry(short index) {
    SCR_RECT rect;
    short      offset = index*12;
    short      loop;

    for (loop=0;loop<2;loop++) {
        SetPlane(loop);
        FillScrRect(&rect,70,offset+14,150,offset+15+10);
        ScrRectFill(&rect,&fullscreen,A_REVERSE);
        DrawStr(ti92_offsetx + 70,
                ti92_offsety + offset + 17,
                settings[index].options[settings[index].active],
                A_REPLACE);
    }
}


/*===========================================================================*/
/* handles complete menu                                                     */
/*===========================================================================*/
short HandleMenu() {
    SCR_RECT rect;
    short      i;
    short      input;
    short      loop;

    FontSetSys(F_6x8);

    //---------------------------------------
    // draw header,footer, menu entries and
    // their selections
    //---------------------------------------
    for (loop=0;loop<2;loop++) {
        SetPlane(loop);
        ClearScreen();
        FillScrRect(&rect,0,0,159,10);
        ScrRectFill(&rect,&fullscreen,A_REPLACE);
        FillScrRect(&rect,0,89,159,99);
        ScrRectFill(&rect,&fullscreen,A_REPLACE);

        for (i=0;i<6;i++) DrawStr(ti92_offsetx + 5,
                                  ti92_offsety+i*12+17,
                                  menu_titles[i],A_REPLACE);

        DrawStr(ti92_offsetx +   5,ti92_offsety +  2, PRG_NAME,           A_XOR);
        DrawStr(ti92_offsetx + 128,ti92_offsety +  2, PRG_VERSION,        A_XOR);
        DrawStr(ti92_offsetx +   4,ti92_offsety + 91, "(c) TI-Chess Team",A_XOR);
        DrawStr(ti92_offsetx + 128,ti92_offsety + 91, "2002",             A_XOR);
    }

    InvertMenuEntry();
    for (i=0;i<5;i++) DrawMenuEntry(i);

    //---------------------------------------
    // menu handling loop
    //---------------------------------------
    do {
        input = GetUserInput();
        if (input == KEY_ESC) return 0;
        if (input == KEY_ENTER) {
            if (selected_entry == 5) return 0;
            return 1;
        }

        //---------------------------------------
        // treatment of KEY_DOWN and KEY_UP:
        // change active menu entry
        //---------------------------------------
        if (input == KEY_DOWN) {
            InvertMenuEntry();
            selected_entry++;
            if (selected_entry>5) selected_entry = 0;
            InvertMenuEntry();
        }
        if (input == KEY_UP) {
            InvertMenuEntry();
            selected_entry--;
            if (selected_entry<0) selected_entry = 5;
            InvertMenuEntry();
        }

        if (selected_entry == 5) continue;

        //---------------------------------------
        // treatment of KEY_LEFT and KEY_RIGHT:
        // select another option of the active
        // menu entry
        //---------------------------------------
        if (input == KEY_LEFT) {
            settings[selected_entry].active--;
            if (settings[selected_entry].active <0)
                settings[selected_entry].active = settings[selected_entry].nr_options-1;
            DrawMenuEntry(selected_entry);
        }
        if (input == KEY_RIGHT) {
            settings[selected_entry].active++;
            if (settings[selected_entry].active >= settings[selected_entry].nr_options)
                settings[selected_entry].active = 0;
            DrawMenuEntry(selected_entry);
        }
    }
    while (1);
}
